<?php

defined( 'ABSPATH' ) or die( 'Keep Silent' );

if ( ! class_exists( 'Beetan_Customize_Typography_Control' ) ):
	class Beetan_Customize_Typography_Control extends WP_Customize_Control {

		public  $type   = 'typography';
		private $placeholder;
		private $extras = array();

		public function __construct( $manager, $id, $args = array() ) {
			parent::__construct( $manager, $id, $args );

			$this->placeholder = isset( $args['placeholder'] ) ? $args['placeholder'] : '';
			$this->extras      = isset( $args['extras'] ) ? $args['extras'] : array();
		}

		public function to_json() {
			parent::to_json();

			$this->json['placeholder'] = $this->placeholder;
			$this->json['extras']      = $this->extras;
		}

		public function enqueue() {
			$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

			wp_enqueue_style( 'select2', esc_url( get_theme_file_uri( "/assets/css/select2{$suffix}.css" ) ), array(), beetan_assets_version( get_theme_file_uri( "/assets/css/select2{$suffix}.css" ) ) );
			wp_enqueue_script( 'select2', esc_url( get_theme_file_uri( "/assets/js/select2{$suffix}.js" ) ), array( 'jquery' ), beetan_assets_version( get_theme_file_uri( "/assets/js/select2{$suffix}.js" ) ), true );

			wp_enqueue_script( 'customize-typography-control', esc_url( get_theme_file_uri( "/assets/js/customize-typography-control{$suffix}.js" ) ), array(
				'jquery',
				'select2'
			), beetan_assets_version( get_theme_file_uri( "/assets/js/customize-typography-control{$suffix}.js" ) ), true );
			wp_enqueue_style( 'customize-typography-control', esc_url( get_theme_file_uri( "/assets/css/customize-typography-control{$suffix}.css" ) ), array(), beetan_assets_version( get_theme_file_uri( "/assets/css/customize-typography-control{$suffix}.css" ) ) );
		}

		protected function render_content() {
			if ( empty( $this->choices ) ) {
				return;
			}
			?>
            <label>
				<?php if ( ! empty( $this->label ) ) : ?>
                    <span class="customize-control-title"><?php echo esc_html( $this->label ); ?></span>
				<?php endif; ?>

                <select <?php $this->link(); ?> class="customize-control-typography-input" style="width: 100%">
					<?php
					foreach ( $this->choices as $values => $labels ) {
						if ( is_array( $labels ) ) {
							printf( '<optgroup label="%s">', esc_attr( $labels['label'] ) );

							foreach ( $labels['options'] as $value => $label ) {
								printf( '<option value="%s" %s>%s</option>', esc_attr( $value ), selected( $this->value(), $value, false ), esc_html( $label ) );
							}

							echo '</optgroup>';
						} else {
							printf( '<option value="%s" %s>%s</option>', esc_attr( $values ), selected( $this->value(), $values, false ), esc_html( $labels ) );
						}
					}
					?>
                </select>

				<?php if ( ! empty( $this->description ) ) : ?>
                    <span class="description customize-control-description"><?php echo wp_kses_post( $this->description ); ?></span>
				<?php endif; ?>
            </label>
			<?php
		}
	}
endif;

if ( ! class_exists( 'Beetan_Customize_Typography' ) ):
	class Beetan_Customize_Typography {

		public function __construct( $wp_customize, $id, $args = array() ) {
			$this->add_settings( $wp_customize, $id, $args );
			$this->add_controls( $wp_customize, $id, $args );
		}

		private function add_settings( $wp_customize, $id, $args = array() ) {

			$wp_customize->add_setting( sprintf( '%s[family]', $id ), array(
				'default'           => isset( $args['default']['family'] ) ? $args['default']['family'] : false,
				'sanitize_callback' => 'sanitize_key',
			) );

			$wp_customize->add_setting( sprintf( '%s[style]', $id ), array(
				'default'           => isset( $args['default']['style'] ) ? $args['default']['style'] : false,
				'sanitize_callback' => 'sanitize_key',
			) );

			$wp_customize->add_setting( sprintf( '%s[subset]', $id ), array(
				'default'           => isset( $args['default']['subset'] ) ? $args['default']['subset'] : false,
				'sanitize_callback' => 'sanitize_key',
			) );

			$wp_customize->add_setting( sprintf( '%s[size]', $id ), array(
				'default'           => isset( $args['default']['size'] ) ? $args['default']['size'] : '0px',
				'sanitize_callback' => 'esc_attr',
			) );
		}

		private function add_controls( $wp_customize, $id, $args ) {

			$wp_customize->add_control( new Beetan_Customize_Typography_Control( $wp_customize, sprintf( '%s[family]', $id ), array(
				'label'       => sprintf( esc_html__( '%s Family', 'beetan' ), $args['label'] ),
				// phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
				'description' => wp_kses_post( isset( $args['description'] ) ? $args['description'] : '' ),
				'placeholder' => esc_html__( 'Font Family', 'beetan' ),
				'section'     => $args['section'],
				'choices'     => array(
					array(
						'label'   => esc_html__( 'Standard Fonts', 'beetan' ),
						'options' => beetan_standard_fonts()
					),
					array(
						'label'   => esc_html__( 'Google Fonts', 'beetan' ),
						'options' => beetan_google_fonts()
					),
				),
				'extras'      => array(
					'id' => $id
				)
			) ) );

			$wp_customize->add_control( new Beetan_Customize_Typography_Control( $wp_customize, sprintf( '%s[style]', $id ), array(
				'label'       => sprintf( esc_html__( '%s Style', 'beetan' ), $args['label'] ),
				// phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
				'description' => wp_kses_post( isset( $args['description'] ) ? $args['description'] : '' ),
				'placeholder' => esc_html__( 'Font Style', 'beetan' ),
				'section'     => $args['section'],
				'choices'     => beetan_standard_font_styles(),
				'extras'      => array(
					'id' => $id
				)
			) ) );

			$wp_customize->add_control( new Beetan_Customize_Typography_Control( $wp_customize, sprintf( '%s[subset]', $id ), array(
				'label'       => sprintf( esc_html__( '%s Subset', 'beetan' ), $args['label'] ),
				// phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
				'description' => wp_kses_post( isset( $args['description'] ) ? $args['description'] : '' ),
				'placeholder' => esc_html__( 'Font Subset', 'beetan' ),
				'section'     => $args['section'],
				'choices'     => beetan_standard_font_styles(),
				'extras'      => array(
					'id' => $id
				)
			) ) );

			$wp_customize->add_control( new Beetan_Customize_Unit_Control( $wp_customize, sprintf( '%s[size]', $id ), array(
				'label'       => sprintf( esc_html__( '%s Size', 'beetan' ), $args['label'] ),
				// phpcs:ignore WordPress.WP.I18n.MissingTranslatorsComment
				'description' => wp_kses_post( isset( $args['description'] ) ? $args['description'] : '' ),
				'placeholder' => esc_html__( 'Font Size', 'beetan' ),
				'section'     => $args['section'],
				'required'    => array(
					sprintf( '%s[family]', $id )
				)
			) ) );
		}
	}
endif;